@@ -1,5 +1,7 @@
 import click
 
+from appollo.helpers import login_required_warning_decorator
+
 
 @click.group('build')
 def build():
@@ -8,10 +10,10 @@ def build():
     \f
     When starting a new project commands are mostly executed in this order :
 
-        1. :code:`appollo build start --build-type="configuration [DIRECTORY]"` to create an Appollo-Remote with the right environment for your application. This will return a key, you will use in the next commands.
-        2. :code:`appollo build connect <key>` to get your connection settings and access the Appolo Remote on which you can test and setup your application with XCode.
-        3. :code:`appollo build stop <key>` to stop your Appolo Remote when you're done editing your settings.
-        4. :code:`appollo build patch <key>` to retrieve the changes made on the Appollo-Remote.
+        1. :code:`appollo build start --build-type="configuration [DIRECTORY]"` to create an Appollo-Remote with the right environment for your application.
+        2. :code:`appollo build connect` to get your connection settings and access the Appolo Remote on which you can test and setup your application with XCode.
+        3. :code:`appollo build stop` to stop your Appolo Remote when you're done editing your settings.
+        4. :code:`appollo build patch` to retrieve the changes made on the Appollo-Remote.
         5. :code:`git am appollo.patch` to apply the changes locally.
         6. :code:`appollo build start [OPTIONS] [DIRECTORY]` to build your app with Flutter and generate an IPA or to publish your app on the App Store.
 
@@ -21,6 +23,7 @@ def build():
 
 
 @build.command()
+@login_required_warning_decorator
 @click.option('-a', '--all', 'show_all', default=False, is_flag=True,
               help="shows your builds and the builds from your teams")
 def ls(show_all):
@@ -67,16 +70,32 @@ def ls(show_all):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 def rm(key):
-    """ Delete build with key \"KEY\" and its corresponding Appollo-Remote.
+    """ Deletes a build and its corresponding Appollo-Remote.
 
     \f
 
     .. note:: Appollo-Remotes are MacOS build machines on which the flutter build of your application is executed.
     """
+    import textwrap
+
+    from rich.text import Text
+
     from appollo import api
     from appollo.settings import console
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", api_params={"all": 1}, name=build_name,
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     build_instance = api.delete(f"/builds/{key}/")
 
@@ -85,9 +104,10 @@ def rm(key):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 def detail(key):
-    """ Fetches detailed information of build with key \"KEY\".
+    """ Fetches detailed information of a build.
 
     Returns the location of the logs and application artifacts.
     """
@@ -99,6 +119,17 @@ def detail(key):
 
     from appollo import api
     from appollo.settings import console
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", api_params={"all": 1}, name=build_name,
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     build_instance = api.get(f"/builds/{key}/")
 
@@ -148,16 +179,18 @@ def detail(key):
 
 
 @build.command()
+@login_required_warning_decorator
+@click.argument('app-key', required=False)
 @click.argument('directory', type=click.Path(exists=True, resolve_path=True, file_okay=False, dir_okay=True),
                 required=False)
-@click.option('--app-key', required=True, prompt=True, help="Key of the application to build")
 @click.option('--build-type', help="Build type", prompt=True,
               type=click.Choice(["configuration", "development", "ad-hoc", "distribution", "validation", "publication"]))
-@click.option('--flutter', help="Flutter version for your build (example \"2.8.1\")",)
+@click.option('--flutter', help="Flutter version for your build (example \"2.8.1\"). Use appollo build flutter-versions to see all available versions",)
 @click.option('--minimal-ios-version', help="Minimal iOS version for you application (example \"9.0\")")
 @click.option('--app-version', help="App version to set for this build (for example \"1.3.1\"). If not set, the version in pubspec.yaml will be used")
 @click.option('--build-number', help="Build number to set for this build (the number after '+' in the version in pubspec.yaml). If not set, the build number in pubspec.yaml will be used")
-def start(app_key, build_type, flutter, minimal_ios_version, app_version, build_number, directory=None):
+@click.option('--no-progress', is_flag=True, help="Do not display the progress and exit the command immediately.")
+def start(build_type, flutter, minimal_ios_version, app_version, build_number, no_progress, app_key=None, directory=None):
     """ Start a new build from scratch
 
     DIRECTORY : Home directory of the flutter project. If not provided gets the current directory.
@@ -180,23 +213,43 @@ def start(app_key, build_type, flutter, minimal_ios_version, app_version, build_
         * **Publication** : Build an .ipa file and publish it on the App Store. Once this build succeeds you have
           to go on the App Store to complete information and screenshots related to your new application version.
 
-    Build start in detached mode you can check the progress of all your Appollo-Remotes by running
-    :code:`appollo build ls` or get detailed information by running :code:`appollo build detail <key>`
+    Killing this command will not stop the build you can check the progress of all your Appollo-Remotes by running
+    :code:`appollo build ls` or get detailed information by running :code:`appollo build detail` and selecting your build.
     """
     import os
+    import textwrap
+    from time import sleep
 
+    from rich.text import Text
     from rich.syntax import Syntax
+
     from appollo.settings import console
-    from appollo.helpers import zip_directory
+    from appollo.helpers import zip_directory, terminal_menu
     from appollo import api
 
     if directory is None:
         directory = os.getcwd()
 
-    print(f"Zipping {directory}")
-    zip_directory(directory)
+    if app_key is None:
+        app_key = terminal_menu("/applications/", "Application",
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have no app identifiers in your account. Check out [code]$ appollo app mk [/code] to create an app identifier.
+                                        """
+                                    )))
+        if app_key is None:
+            return
 
-    print(f"Uploading {directory}")
+    console.print(f"Zipping {directory}")
+    zip_file = zip_directory(directory)
+
+    file_size_mb = round(os.path.getsize(zip_file)/1000000, 2)
+
+    if file_size_mb > 1000:
+        console.print("File size exceeds 1GB, very large applications are not supported by Appollo.")
+        return
+
+    console.print(f"Uploading {directory} ({file_size_mb} MB)")
 
     build_instance = api.post(
         "/builds/",
@@ -222,21 +275,85 @@ def start(app_key, build_type, flutter, minimal_ios_version, app_version, build_
             code = Syntax(code=f"appollo build connect {build_instance['key']}", lexer="shell")
             console.print("To access your Appollo-Remote, you can use the following command when it has been started.")
             console.print(code)
+        console.print("Killing the command will not stop the build.")
 
-    # TODO show realtime logs and build process.
+        if no_progress:
+            return
+
+        # check build progress.
+        loop = True
+        with console.status("Waiting for available instance...", spinner="line") as spinner:
+            while loop:
+                # update status every 10 second.
+                sleep(10)
+                build_instance = api.get(f"/builds/{build_instance['key']}/")
+
+                status = build_instance["status_code"]
+                if status == "created" or status == "waiting_instance":
+                    spinner.update("Waiting for available instance...")
+                elif status == "in_progress":
+                    substatus = build_instance["substatus_code"]
+                    if substatus == "starting_instance":
+                        spinner.update("Starting instance...")
+                    elif substatus == "preparing_build":
+                        spinner.update("Preparing build...")
+                    elif substatus == "getting_result":
+                        spinner.update("Getting result...")
+                    elif substatus == "publishing":
+                        spinner.update("Publishing...")
+                    else:
+                        spinner.update("Building...")
+                elif status == "config":
+                    spinner.update("Configured for remote access")
+                    loop = False
+                elif status == "succeeded":
+                    spinner.update("Success!")
+                    loop = False
+                elif status == "failed":
+                    spinner.update("Failed")
+                    loop = False
+                elif status == "stopped":
+                    spinner.update("Stopped")
+                    loop = False
+
+        if status in ["config", "succeeded"]:
+            console.print(Text.from_markup(f"Your build has succeeded"))
+            if status == "config":
+                console.print("You can access your Appollo-Remote by running the following command.")
+                console.print(code)
+            return
+        elif status in ["failed", "stopped"]:
+            console.print(Text.from_markup(f"Your build has failed, to access logs run : [code]appollo build logs {build_instance['key']}[/code]"))
+            return
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 def ipa(key):
-    """ Get the ipa file of build with key \"KEY\".
+    """ Get the ipa file of a build.
 
     \b
     This command is used when a build of type ad-hoc, ipa, validation or publication has succeeded.
     It returns an url to get the IPA, either to download it, or to install it if opened from an iOS device.
     """
+    import textwrap
+
+    from rich.text import Text
+
     from appollo import api
     from appollo.settings import console
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", api_params={"all": 1}, name=build_name,
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     response = api.get(f"/builds/{key}/ipa/")
 
@@ -248,18 +365,34 @@ def ipa(key):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 @click.option(
-    "-o", "--output", default="result.zip", help="Output filename (default: result.zip)",
+    "-o", "--output", default="source.zip", help="Output filename (default: source.zip)",
     type=click.Path(exists=False, resolve_path=True, file_okay=True, dir_okay=False))
-def result(key, output="result.zip"):
-    """ Get the modified sources of build with key \"KEY\".
+def download(key, output="source.zip"):
+    """ Get the modified source code of a build.
 
     \b
-    This command is used when a build is finished to retrieved its entire directory on the remote machine.
+    This command is used when a build is finished or stopped to retrieve its entire directory on the remote machine.
     """
+    import textwrap
+
+    from rich.text import Text
+
     from appollo import api
     from appollo.settings import console
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", api_params={"all": 1}, name=build_name,
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     response = api.get(f"/builds/{key}/result/", json_decode=False)
 
@@ -267,14 +400,15 @@ def result(key, output="result.zip"):
         with open(output, "wb") as f:
             f.write(response)
     else:
-        console.print("This build does not have any result")
+        console.print("Could not download the source code. Are you sure the build is stopped ?")
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 @click.option("-y", "--yes", is_flag=True, help="Automatically create an Appollo Remote if your build was not setup for remote desktop", )
 def connect(key, yes):
-    """ Get the connection information for an Appollo-Remote linked to build with key \"KEY\".
+    """ Get the connection information for an Appollo-Remote linked to a build.
 
     \b
     This command is mainly used when it is needed to connect to the Appollo-Remote with a Remote Desktop client :
@@ -287,8 +421,9 @@ def connect(key, yes):
     This command will start an Appollo-Remote and return connection settings and credentials for you to connect with a
     remote desktop client to the Appollo-Remote.
 
+    .. note:: Appollo-Remotes are active for 30 minutes before being closed automatically.
     .. note:: Appollo-Remotes are MacOS build machines on which the flutter build of your application is executed.
-    .. note:: The connection uses spice protocol, your Remote DEsktop must support it to allow you to use an Appollo Remote
+    .. note:: The connection uses the spice protocol, your Remote Desktop client must support it to allow you to use an Appollo-Remote.
     """
     import textwrap
 
@@ -297,9 +432,19 @@ def connect(key, yes):
     from rich.syntax import Syntax
 
     from appollo.settings import console
-
+    from appollo.helpers import terminal_menu
     from appollo import api
 
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", name=build_name,
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
+
     build_instance = api.get(f"/builds/{key}/connect/")
 
     if build_instance:
@@ -320,7 +465,7 @@ def connect(key, yes):
                     console.print(code)
 
         elif build_instance["remote_desktop_status"] == "remote_desktop_preparation":
-            console.print("Your VM is currently prepared for being used as Remote Desktop. Please try again in a few moments.")
+            console.print("Your Appollo-Remote is currently prepared for being used as Remote Desktop. Please try again in a few moments.")
         else:
             auth_info = Panel(Text.from_markup(
                 textwrap.dedent(
@@ -349,23 +494,39 @@ def connect(key, yes):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 @click.option(
     "-o", "--output", default="appollo.patch", help="Output filename (default: appollo.patch)",
     type=click.Path(exists=False, resolve_path=True, file_okay=True, dir_okay=False))
-def patch(key, output="result.zip"):
-    """ Retrieve a patch that gathers all changes made to the source code of build with key \"KEY\".
+def patch(key, output="appollo.patch"):
+    """ Retrieve a patch that gathers all changes made to the source code of a build.
 
     \b
     This command is used when a build is finished and succeeded or when you have stopped an Appollo Remote on which you configured an app with XCode.
 
     .. note:: The patch was made on a blank Git repo.
     """
+    import textwrap
+
+    from rich.text import Text
     from rich.syntax import Syntax
 
     from appollo.settings import console
     from appollo import api
+    from appollo.helpers import terminal_menu
 
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", name=build_name, api_params={"all": 1},
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
+
+    console.print(f"Downloading patch file as {output}.")
     response = api.get(f"/builds/{key}/patch/", json_decode=False)
 
     if response:
@@ -380,13 +541,27 @@ def patch(key, output="result.zip"):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 def stop(key):
-    """ Stops running build KEY """
+    """ Stops a running build"""
+    import textwrap
+
     from rich.text import Text
 
     from appollo.settings import console
     from appollo import api
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", name=build_name, api_params={"all": 1},
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     build_instance = api.post(f"/builds/{key}/stop/")
     if build_instance:
@@ -396,17 +571,57 @@ def stop(key):
 
 
 @build.command()
-@click.argument('key', required=True)
+@login_required_warning_decorator
+@click.argument('key', required=False)
 def logs(key):
-    """
-    Outputs the logs of build KEY
+    """ Outputs the logs of a build
 
+    \f
     .. note:: Logs are printed only when a command has finished its execution. In particular, Flutter logs are only printed when the flutter command execution has ended.
     """
+    import textwrap
+
+    from rich.text import Text
+
     from appollo.settings import console
     from appollo import api
+    from appollo.helpers import terminal_menu
+
+    if key is None:
+        key = terminal_menu("/builds/", "Builds", name=build_name, api_params={"all": 1},
+                                    does_not_exist_msg=Text.from_markup(textwrap.dedent(
+                                        f"""
+                                            You have not run any builds yet.
+                                        """
+                                    )))
+        if key is None:
+            return
 
     logs = api.get(f"/builds/{key}/logs/")
     console.print(logs)
 
-# TODO Commandes pour lister les versions Flutter disponibles
+
+def build_name(build_instance):
+    """ Based on a build instance returns a user friendly name for the build. """
+    from datetime import datetime
+
+    start_time = datetime.strptime(build_instance['start_time'], "%Y-%m-%dT%H:%M:%S.%f%z")
+    start_time_str = start_time.strftime('%Y-%m-%d %H:%M')
+    # TODO check if timezones are respected.
+    return f"{build_instance['application']} - {build_instance['name']} - {start_time_str}"
+
+
+@build.command()
+@login_required_warning_decorator
+def flutter_versions():
+    """ Lists the Flutter versions available on Appollo. """
+    from rich.columns import Columns
+
+    from appollo import api
+    from appollo.settings import console
+
+    versions = api.get("/flutter-versions/")
+
+    if versions:
+        columns = Columns(versions, padding=(0, 3), equal=True, title="Flutter versions available")
+        console.print(columns)
